@echo off

rem RENAME THIS FILE TO FTP_Reprocesses.bat and store it in the C:\CAMS directory.
rem
rem This script needs to be modified (or a new script made) that allows batch
rem processing of multiple dates and/or processing of a date that is a date 
rem some other than the current date.
rem
rem It should be modified to run the check like ValidateCal does to ensure
rem that the Mean O-C number is less than 1.0.  It can do this by checking
rem the first two digits of the Mean O-C value for "0.".  (See ValidateCal.bat)
rem
rem It shold also check for valid .bin files like CheckBinFileSize.bat does. 
rem
rem It could also check that another ftp_reprocess.exe script is not already
rem running (because the process overwrites and will have a conflict with
rem some files in x:\cams. 
rem
rem Here is how this will be called from within FTP_Capture.bat:
rem 	echo  %date% %time% "FTP_Reprocess start" "!ftp_captureddir!" %1 %2 %3 %4>>"%~0.log"
rem 	title FTP_Reprocess
rem 	call "ftp_reprocess.bat" 1 "!ftp_captureddir!" "%3\CAMS\Cal"
rem			Where: 1                    - means copy ff*.bin files to ArchivedFiles
rem                "!ftp_captureddir!"  - is something like: 
rem                                         "h:\CAMS\CapturedDir\2011_08_21_20_30_43"
rem                "%3\CAMS\Cal"        - is something like:
rem                                         "h:\CAMS\Cal"
rem
rem 	echo  %date% %time% "FTP_Reprocess end" "!ftp_captureddir!" %1 %2 %3 %4>>"%~0.log"
rem
rem You should remember that the ArchivedFiles\datedir does not have the same date as
rem the CaptureFiles\datedir.  The datedir of CapturedFiles is the date/time when
rem FTP_Capture.exe is started, whereas the datedir of ArchivedFiles is the date/time
rem of the first file found in the CaptureFiles directory (that FTP_Reprocess.exe opens
rem and starts processing). Note: All date/times are in UTC.  Therefore, unless it is 
rem starting before 4pm (00:00:00 UTC), the date will actually have a value of
rem the following day.
rem 
rem TIPs: 
rem  This command returns the line that has the file count
rem   dir "C:\CAMS\SubmissionFiles\2011_08_04_06_04_11\ArchivedFiles\2011_08_04_06_04_11\*.bin"|find /i "file(s)"
rem Then, if you do  this command, using one less than the total files, you
rem will be able to get the first file name.
rem   for /f "usebackq tokens=* skip=360" %a in (`dir "C:\CAMS\SubmissionFiles\2011_08_04_06_04_11\ArchivedFiles\2011_08_04_06_04_11\*.bin" /s/b /o-d`) do (echo %a)
rem 

:SETUP
	setlocal enableextensions enabledelayedexpansion

:MAIN
	rem start the execution of the detection pass in background priority

	echo %date% %time%>"%~0.log"


	rem Note: 
	call :date2jdate ftp_jd %ftp_year% %ftp_mo2% %ftp_da2%
	rem set /a ftp_jd=%ftp_jd%-1
	call :jdate2date %ftp_jd% ftp_year ftp_mo2 ftp_da2

	set ftp_searchdir=%ftp_year%_%ftp_mo2%_%ftp_da2%


	set ftp_captureddir=%~1

	set ftp_

	rem Note: All date/times are in UTC.  Therefore, unless it is starting 
	rem before 4pm (00:00:00 UTC), the date will actually have a value of
	rem the following day.
	set /a ftp_linecount=0
	for /f "usebackq tokens=*" %%a in (`dir "!ftp_captureddir!\*.bin" /s/b /a-d^|sort /reverse`) do (
		set ftp_archiveddir=%%~nxa
	)

	set ftp_

	if defined ftp_archiveddir (
		dir ".\ArchivedFiles\%ftp_searchdir%*.*" /s/b /od /ad>"%~0.log"
		dir ".\ArchivedFiles\%ftp_searchdir%*.*" /s/b /od /ad
		goto :ERR1
	)

:SRCHDRIVEC
	for /f "usebackq tokens=*" %%a in (`dir "c:\cams\ArchivedFiles\%ftp_searchdir%*.*" /s/b /od /ad`) do (
		echo %%~nxa
		rem set ftp_temp=%%~nxa
		set ftp_archiveddir=%%~nxa
	)

	set ftp_

	if defined ftp_archiveddir (
		dir "C:\cams\ArchivedFiles\%ftp_searchdir%*.*" /s/b /od /ad>"%~0.log"
		dir "C:\cams\ArchivedFiles\%ftp_searchdir%*.*" /s/b /od /ad
		goto :ERR1
	)

:CONTINUE


	@echo start "FTP_Reprocess" /wait ".\ftp_reprocess.exe" ".\CapturedFiles\%ftp_captureddir%">>"%~0.log"
	 echo start "FTP_Reprocess" /wait ".\ftp_reprocess.exe" ".\CapturedFiles\%ftp_captureddir%"
goto :DONE

:ERR1
	@echo.
	@echo.
	@echo "%cd%\ArchivedFiles\%ftp_archiveddir%" already exists. Do you want to
	@echo continue to overwrite the existing files? [Y/N] Y
	@goto :CONTINUE
	@set /p retval=continue to overwrite the existing files? [Y/N] 
	@if /i "%retval%" equ "y" goto :CONTINUE
goto :DONE


:date2jdate JD YYYY MM DD -- converts a gregorian calender date to julian day format
	setlocal
	rem                        -- JD   [out] - julian days
	rem                        -- YYYY [in]  - gregorian year, i.e. 2006
	rem                        -- MM   [in]  - gregorian month, i.e. 12 for december
	rem                        -- DD   [in]  - gregorian day, i.e. 31
	rem :$reference http://aa.usno.navy.mil/faq/docs/JD_Formula.html
	rem :$created 20060101 :$changed 20080219 :$categories DateAndTime
	rem :$source http://www.dostips.com
	set "yy=%~2"&set "mm=%~3"&set "dd=%~4"
	set /a "yy=10000%yy% %%10000,mm=100%mm% %% 100,dd=100%dd% %% 100"
	if %yy% LSS 100 set /a yy+=2000 &rem Adds 2000 to two digit years
	set /a JD=dd-32075+1461*(yy+4800+(mm-14)/12)/4+367*(mm-2-(mm-14)/12*12)/12-3*((yy+4900+(mm-14)/12)/100)/4
	ENDLOCAL & IF "%~1" NEQ "" (SET %~1=%JD%) ELSE (echo.%JD%)
EXIT /b

:jdate2date JD YYYY MM DD -- converts julian days to gregorian date format
	rem                     -- JD   [in]  - julian days
	rem                     -- YYYY [out] - gregorian year, i.e. 2006
	rem                     -- MM   [out] - gregorian month, i.e. 12 for december
	rem                      -- DD   [out] - gregorian day, i.e. 31
	rem :$reference http://aa.usno.navy.mil/faq/docs/JD_Formula.html
	rem :$created 20060101 :$changed 20080219 :$categories DateAndTime
	rem :$source http://www.dostips.com
	SETLOCAL ENABLEDELAYEDEXPANSION
	set /a L= %~1+68569,     N= 4*L/146097, L= L-(146097*N+3)/4, I= 4000*(L+1)/1461001
	set /a L= L-1461*I/4+31, J= 80*L/2447,  K= L-2447*J/80,      L= J/11
	set /a J= J+2-12*L,      I= 100*(N-49)+I+L
	set /a YYYY= I,  MM=100+J,  DD=100+K
	set MM=%MM:~-2%
	set DD=%DD:~-2%
	( ENDLOCAL & REM RETURN VALUES
	    IF "%~2" NEQ "" (SET %~2=%YYYY%) ELSE echo.%YYYY%
	    IF "%~3" NEQ "" (SET %~3=%MM%) ELSE echo.%MM%
	    IF "%~4" NEQ "" (SET %~4=%DD%) ELSE echo.%DD%
	)
EXIT /b

:DONE
	echo %date% %time%>>"%~0.log"
