   @ECHO OFF

set RetVal=
set RetVal1=
set RetVal2=
SETLOCAL ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION


set local.quietmode=false
set batchfilename=%~f0

:SETUP
	rem Check Windows version
	IF NOT "%OS%"=="Windows_NT" GOTO :HELP "!batchfilename!"

	rem Check command line
	ECHO.%* | FIND "?" >NUL
	IF NOT ERRORLEVEL 1 GOTO :HELP "!batchfilename!"

	rem Read command line arguments
	set local.quietmode=false
	set /a arg=0
:ARGLOOP
	set /a arg+=1
	if "%~1" equ "" goto :ARGLOOPEND
	echo.%~1|find /i "/q">nul
	if %errorlevel% equ 0 (
		set local.quietmode=true
		shift
		goto :ARGLOOP
	)
	
	if %arg% equ 1 (
		set INIFile="%~f1"
		shift
		GOTO :ARGLOOP
	)
	if %arg% equ 2 (
		set INISection=%~1
		shift
		GOTO :ARGLOOP
	)
	if %arg% equ 3 (
		set INIKey=%~1
		SET INIValue=
		shift
		GOTO :ARGLOOP
	)
	goto :ARGLOOP
:ARGLOOPEND
	
	if %local.quietmode% equ false (
		for /f "usebackq" %%a in ('!batchfilename!') do (
			echo Batch script: %%~nxa
			echo.
		)
	)

	IF DEFINED INIKey (
		set local.listwholesection=false
	) else (
		set local.listwholesection=true
	)
	
	SET SkipLine=goto :EOF

:MAIN
	rem Reset temporary variables
	SET SectComplete=0
	SET SectFound=0
	SET KeyFound=0
	
	rem  Search the INI file line by line
	if not exist %INIFile% (
		echo.
		echo ERRORLEVEL 4: File !INIFile! not found.
		EXIT /B 4
	)
	
	FOR /F "tokens=* delims=" %%A IN ('TYPE !INIFile!') DO (
		CALL :ParseINI "%%A"
	)
	
	goto :DISPLAYRESULT
	

:DISPLAYRESULT
	rem  Display the result
	
	rem EXIT /B return codes (errorlevels) added by Van Woods,
	rem  US Army Corps of Engineers, Seattle District
	IF %SectFound%==0 (
		ECHO !INIFile!
		ECHO ERRORLEVEL 1: Section [%INISection%] not found in !INIFile!.
		EXIT /B 1
	) else (
		if !local.listwholesection! equ true (
			exit /B 0
		)
		
		IF NOT %KeyFound%==1 (
			ECHO !INIFile!
			ECHO !SaveSection!
			ECHO ERRORLEVEL 2: Key "%INIKey%" not found in !INIFile!, section !SaveSection!.
			echo.
			echo Note: Return values are in RetVal environment variables RetVal, RetVal1
			( ENDLOCAL
					set RetVal1=%INIValue%
					set RetVal=%key%=%inivalue%
			)
			EXIT /B 2
		) eLse (
			IF DEFINED INIValue (
				ECHO !INIFile!
				ECHO !SaveSection!
				ECHO !Key!=!INIValue!
				echo.
				echo Note: Return values are in RetVal environment variables RetVal, RetVal1
				( ENDLOCAL
					set RetVal1=%INIValue%
					set RetVal=%key%=%inivalue%
				)
				exit /B 0
			) elSe (
				ECHO !INIFile!
				ECHO !SaveSection!
				ECHO !Key!=!INIValue!
				ECHO The !SaveSection! section and Key "%INIKey%=" were found, but 
				echo the value is blank. The %%errorlevel%% code is set to 3, but 
				echo the RetVal1 environment variable will be absent. This is not 
				echo always an invalid INI file condition.
				echo.
				echo Note: Return values are in RetVal environment variables RetVal, RetVal1
				( ENDLOCAL
					set RetVal1=%INIValue%
					set RetVal=%key%=%inivalue%
				)
				EXIT /B 3
			)
		)
	)
GOTO :DONE


:ParseINI
	rem echo :PARSEINI sectfound=%sectfound%, keyfound=%keyfound%, sectcomplete=%sectcomplete%, %local.listwholesection%
	rem  Skip rest of file after key has been found;
	IF "%SectFound%"=="1" (
		if "%local.listwholesection%" equ "false" (
			IF "%KeyFound%"=="1" %SKIPLINE%
		)
	)
	
	rem  Store quoted line in variable
	SET Line="%~1"
	set SaveLine=%~1
	
	rem  Check if this line is the required section heading
	ECHO.%Line%| FIND /I "[%INISection%]" >NUL
	if "%SectFound%" neq "1" (
		ECHO.%Line%| FIND /I "[%INISection%]" >NUL
		IF %ERRORLEVEL% equ 0 (
			SET SectFound=1
			set SaveSection=!SaveLine!
			if !local.listwholesection! equ true (
				ECHO.!INIFile!
				ECHO !SaveSection!
			)
		)
		goto :EOF
	)

	if "%SectFound%" equ "1" (
		rem  Check if this line is a different section header
		IF "%Line:~1,1%"=="[" (
			SET SectComplete=1
		)
		
		IF "!SectComplete!"=="1" (
			%SKIPLINE%
		)

		rem  Parse any "key=value" line
		FOR /F "tokens=1* delims==" %%a IN ('ECHO.!SaveLine!') DO (
			SET Key=%%a
			SET Value=%%b
		)

		if /i %local.listwholesection% equ true (
			echo !Key!=!Value!
		)

		rem  Now check if the key matches the required key
		IF /I "!Key!"=="%INIKey%" (
			SET INIValue=!Value!
			SET KeyFound=1
		)
	)
GOTO:EOF

:TRIMRHT
:loop2
	 if "!trimmedstring:~-1!" equ " " (
		 set trimmedstring=!trimmedstring:~0,-1!
	 ) else (
	 	goto :BREAK
	 )
	 goto loop2

:BREAK
goto :EOF

:HELP
	ECHO %~nx0,  Version 2.00 for Windows NT 4 / 2000 / XP / Server 2003
	ECHO.
	ECHO Reads a value from the specified INI file and [section]. 
	ECHO.
	ECHO Usage:  %~n0  "ini_file"  "section"  ["key"] [/q]
	echo.
	ECHO Where:           "ini_file" is the file name of the INI file to be read
	ECHO                  "section"  is the section name, without the brackets
	ECHO                  "key"      is the key whose value must be read. If 
	echo                             key is omitted, it lists all key=value pairs
	echo                             in the section.
	echo                  /q         means quiet mode. For use in redirecting to 
	echo                             an output file.
	ECHO.
	echo. RETURNS:
	echo             %%errorlevel%%   1=specified section not found
	echo             %%errorlevel%%   2=specified key not found
	echo             %%errorlevel%%   3=Key found, but no value. This isn't always an error.
	echo.
	ECHO Example 1: If MYPROG.INI looks like this:
	ECHO     [Section 1]
	ECHO     Key1=Value 1
	ECHO     Key2=Value 2
	echo.
	ECHO     [Section 2]
	ECHO     Key2=Value 4
	echo.
	ECHO Then the command line:  
	echo     %~n0 "MYPROG.INI" "section 1" "key2"
	echo.
	ECHO would display on the console:
	echo     Batch script: readini.bat
	echo.
	echo     "C:\...\MYPROG.INI"
	echo     [Section 1]
	echo     Key2=Value 2
	echo.
	echo Example 2: This command line (omitting the Key) would result in the following:
	echo     %~n0 "MYPROG.INI" "section 1"
	echo.
	echo would display this on the console:
	echo     Batch script: readini.bat
	echo.
	echo     "C:\...\MYPROG.INI"
	echo     [Section 1]
	echo     Key1=Value 1
	echo     Key2=Value 2
	echo.
	echo Also, the environment variable will be set using:
	echo     SET RetVal=Key2=Value 2
	echo     SET RetVal1=Value 2
	echo.
	echo     You can check for the results using the following commands:
	echo        set retval
	echo        echo %%errorlevel%%
	ECHO.
goto :DONE

:DONE
